//-----------------------------------------------------------------------------
//
// Source File Name: pwQMSecExit.c
// Version: 1.0
// Description: 
//
// This is a sample configuration manager queue manager security exit 
// which should be used in conjuntion with the sample Control Center
// security exit pwCCSecExit.java
//
//-----------------------------------------------------------------------------
     
     
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <windows.h>
#include <lm.h>
#include <cmqc.h>
#include <cmqxc.h>


              

void MQStart() {;} /*dummy entry point - for consistency only*/

void MQENTRY CHANNELEXIT (  PMQCXP  pChannelExitParms,
                            PMQCD   pChannelDefinition,
                            PMQLONG pDataLength,
                            PMQLONG pAgentBufferLength,
                            PMQVOID pAgentBuffer,
                            PMQLONG pExitBufferLength,
                            PMQPTR  pExitBufferAddr)

{

 
PMQCXP  pParms = pChannelExitParms;
PMQCD   pChDef = pChannelDefinition;

/*======================*/
    
 
  //check if security exit has been invoked 
  if (pParms->ExitId==MQXT_CHANNEL_SEC_EXIT)
  {
   //Exit Buffer not used by java MQChannelExit interface so only use AgentBuffer
   //to transfer data between this queue manager exit and the Control Center 
   //exit
      
   // now check reason for exit invocation 
   switch (pParms->ExitReason)
   {
     case MQXR_INIT:
                     pParms->ExitResponse = InitSExit(pChannelExitParms,
                                                      pChannelDefinition,
                                                      pDataLength,
                                                      pAgentBufferLength,
                                                      pAgentBuffer);
                     break;
     case MQXR_INIT_SEC:
                     //this should be a Control Center exit initiated
                     //exchange therefore this queue manager exit should never
                     //be invoked with MQXR_INIT_SEC
                     printf("ERROR!! - Unexpectedly invoked with MQXR_INIT_SEC - closing channel\n");
                     pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION; 
                     break;
     case MQXR_SEC_MSG:
                     pParms->ExitResponse = receiveSecMsg(pChannelExitParms,
                                                          pChannelDefinition,
                                                          pDataLength,
                                                          *pAgentBufferLength,
                                                          pAgentBuffer
                                                         );
                     break;
     case MQXR_TERM:
                     //reset exit user area and return MQXCC_OK at exit termination
                     memset(pParms->ExitUserArea,0,sizeof(pParms->ExitUserArea));
                     pParms->ExitResponse = MQXCC_OK; 
                     break;
     default: 
                     printf("ERROR!! Invoked with unexpected exitreason");
                     pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
                     break; 
    }
   }
   else
   {
     // if not security exit then exit and close channel
     printf("ERROR!! - Unexpected invocation");   
     pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
     return;
   }
return;
}


MQLONG InitSExit(PMQCXP     I_pCEParms,
                 PMQCD      I_pCDefinition,
                 PMQLONG    I_pDLength,
                 PMQLONG    I_pABufferLength,
                 PMQVOID    I_pABuffer )
{
 
 //reset exituserarea
 memset(I_pCEParms->ExitUserArea,0,sizeof(I_pCEParms->ExitUserArea));
 //return MQXCC_OK to complete exit initialization 
 return MQXCC_OK;

}





MQLONG receiveSecMsg(PMQCXP     pCEParms,
                     PMQCD      pCDefinition,
                     PMQLONG    pDLength,
                     MQLONG     ABufferLength,
                     PMQVOID    pABuffer)
{
    
 //data to put in return message to Control Center exit*/
 MQBYTE ExitData[] = "userid received";
  
 char validID[50];
 //enter server name - eg local machine or PDC
 char validSV[50] = "servername";
 char validPW[50];
 
 char *vvalidID;
 char *vvalidSV;
 char *vvalidPW;

 char pswdpref[] = "pswd";
 char usidpref[] = "usid";
 const int prefixLength = 4;


 DWORD nStatus;
 DWORD Err;
 HANDLE	phToken = NULL;

 int length;
 int rcode = 1;
 
 //set length to size of data in pABuffer area
 length = *pDLength;
 
 vvalidID = validID;
 vvalidPW = validPW;
 vvalidSV = validSV;

 //reset userid and password storage areas
 memset(validID,0,sizeof(validID));
 memset(validPW,0,sizeof(validPW));

 //if no security data sent by Control Center exit then close channel
 if (*pDLength == 0 ) 
 {
  printf ("ERROR!! - No security message received in Agent Buffer - closing channel \n");
  return MQXCC_SUPPRESS_FUNCTION;
 }
 else
 {
     //check if password or userid sent
     rcode = strncmp(pABuffer,usidpref,prefixLength);
     //if userid received in agentBuffer then store in exituserarea
     if ( rcode == 0 )
     {  
        strncpy(pCEParms->ExitUserArea,pABuffer,length);
        //send confirmation message back in agentbuffer to Control Center exit program
        memset(pABuffer,0,length);
        //make sure data fits in agentBuffer
        if ( sizeof(ExitData) <= ABufferLength) {
           memcpy(pABuffer,ExitData,sizeof(ExitData));
           *pDLength = sizeof(ExitData); 
           pCEParms->ExitResponse2 = MQXR2_USE_AGENT_BUFFER;
            
        }
        return MQXCC_SEND_AND_REQUEST_SEC_MSG;
     }
     else
     {  //if password received in agentBuffer
        rcode = strncmp(pABuffer,pswdpref,prefixLength);
        if (rcode == 0)
        {
           //get userid stored in exituserarea
           strncpy(vvalidID,pCEParms->ExitUserArea,sizeof(pCEParms->ExitUserArea));
           strncpy(vvalidPW,pABuffer,length);
           //use LogonUser to verify userid and password
           nStatus = LogonUser(vvalidID+prefixLength,vvalidSV,vvalidPW+prefixLength, LOGON32_LOGON_INTERACTIVE, LOGON32_PROVIDER_DEFAULT,&phToken);
           if (nStatus == 0)
           {
              //if userid/password not verified then close channel
              Err = GetLastError();
              printf("ERROR!! - Logon error Err = %d \n", Err);
              printf ("ERROR!! - Userid not verified - closing channel\n");
              return MQXCC_SUPPRESS_FUNCTION; 
           }
           else
           {  
              //if MCAUserID not set then set it to verified client id
              if (pCDefinition->MCAUserIdentifier[0]  == ' ' ||
                       pCDefinition->MCAUserIdentifier[0]  == '\0')
              {
                 memset(pCDefinition->MCAUserIdentifier,0,sizeof(pCDefinition->MCAUserIdentifier));
                 strncpy(pCDefinition->MCAUserIdentifier,validID+prefixLength,(sizeof(validID)-prefixLength));
              }
              //reset paBuffer
              memset(pABuffer,0,length);
              //return MQOCC_OK to end message exchange
              return MQXCC_OK; 
           }  

        } 
        else
        {
           //if expected userid/password data not sent then close channel
           printf("ERROR!invalid request - closing channel\n");
           //reset paBuffer
           memset(pABuffer,0,length);
           return MQXCC_SUPPRESS_FUNCTION;
        } 
     }
 }


}




